import csv 

def load_employee():
    reader = csv.DictReader(open('travaho.csv','r'))
    employees = dict()
    for line in reader:
        emp = {'name' : line['nom'],
               'salary' : line['salaire'],
               'boss' : line['nom de superieur'] if line['nom de superieur'] != '' else None}
        employees[emp['name']] = emp

    return employees

def find_root(employees):
    for name in employees:
        if not employees[name]['boss']:
            return name

def find_children(employees, boss_name):
    children = set()
    for name in employees:
        if employees[name]['boss'] == boss_name:
            children.add(name)
    return children

def build_tree(employees, name=None):
    if name is None:
        name = find_root(employees)
    return {
        'name' : name,
        'salary' : float(employees[name]['salary']),
        'children' : [
            build_tree(employees, child_name)
            for child_name in find_children(employees, name)
        ]
    }

if __name__ == '__main__' :
    assert find_root(load_employee()) == 'Adhemar Patamob'
    assert find_children(load_employee(), 'Adhemar Patamob') == set(['Agathe Zeublouze', 'Alex Kuzbidon', 'Anasthasie Locale', 'Armand Teutmaronne'])


def sum_desc_salaries(tree):
    s = 0
    for child in tree['children']:
        s += child['salary']
        s += sum_desc_salaries(child)
    return s

def print_tree(tree, ind=0):
    print "   "*ind, tree['name'], 'salary', tree['salary'], 'children', sum_desc_salaries(tree)
    for child in tree['children']:
        print_tree(child, ind+1)



if __name__ == '__main__':
    print_tree(build_tree(load_employee()))
    print sum_desc_salaries(build_tree(load_employee()))

#
# tree = { 
#      'name' : "Adhemar Patamob",
#      'salary' : 50000.0,
#      'children' : [ 
#            { 'name' : "Agathe Zeublouze", 
#              'salary' : 35000.0,
#              'children' : [...]},
#            { 'name' : "Alex Kuzbidon", 
#              'salary' : 34000.0,
#              'children' : [...]}
#      ]
# }     
